local addonName, ns = ...  -- Namespace

local List = ns.List

local Table = {}
ns.Table = Table


---@param parent Frame
---@param header table
---@param highlightHover boolean
---@param highlightClick boolean
function Table:new(parent, highlightHover, highlightClick)
    local o = {}
    setmetatable(o, self)
    self.__index = self

    local mainFrame = CreateFrame('Frame', parent:GetName() .. 'TableFrame', parent)

    mainFrame.scrollFrame = CreateFrame(
        'ScrollFrame',
        mainFrame:GetName() .. 'ScrollFrame',
        mainFrame,
        'UIPanelScrollFrameTemplate'
    )
    mainFrame.scrollFrame:SetPoint('TOPLEFT', mainFrame, 'TOPLEFT', 0, 0) -- TODO: y offset if header
    mainFrame.scrollFrame:SetWidth(mainFrame:GetWidth())
    mainFrame.scrollFrame:SetPoint('BOTTOM', mainFrame, 'BOTTOM', 0, 0)

    local scrollFrameName = mainFrame.scrollFrame:GetName()
    mainFrame.scrollBar = _G[scrollFrameName .. 'ScrollBar'];
    mainFrame.scrollUpButton = _G[scrollFrameName .. 'ScrollBarScrollUpButton'];
    mainFrame.scrollDownButton = _G[scrollFrameName .. 'ScrollBarScrollDownButton'];

    mainFrame.scrollUpButton:ClearAllPoints();
    mainFrame.scrollUpButton:SetPoint('TOPRIGHT', mainFrame.scrollFrame, 'TOPRIGHT', -2, 0);

    mainFrame.scrollDownButton:ClearAllPoints();
    mainFrame.scrollDownButton:SetPoint('BOTTOMRIGHT', mainFrame.scrollFrame, 'BOTTOMRIGHT', -2, -2);

    mainFrame.scrollBar:ClearAllPoints();
    mainFrame.scrollBar:SetPoint('TOP', mainFrame.scrollUpButton, 'BOTTOM', 0, 0);
    mainFrame.scrollBar:SetPoint('BOTTOM', mainFrame.scrollDownButton, 'TOP', 0, 0);

    mainFrame.scrollChild = CreateFrame('Frame')
    mainFrame.scrollChild:SetSize(mainFrame.scrollFrame:GetWidth() - mainFrame.scrollBar:GetWidth() - 7, 1)

    mainFrame.scrollFrame:SetScrollChild(mainFrame.scrollChild);

    -- -- header
    -- if header then
    --     mainFrame.header = CreateFrame('Frame', nil, mainFrame)
    --     mainFrame.header:SetPoint('TOPLEFT', mainFrame, 'TOPLEFT', 2, 0)
    --     mainFrame.header:SetHeight(10)
    --     mainFrame.header:SetPoint('RIGHT', mainFrame.scrollBar, 'LEFT', -5, 0)

    --     mainFrame.header.columns = {}

    --     local columnWidth = mainFrame.header:GetWidth() / #header

    --     for i, headerCol in ipairs(header) do
    --         local headerText = headerCol[1]
    --         local justify = headerCol[2]

    --         -- local xOffset = (i - 1) * columnWidth

    --         local column = mainFrame.header:CreateFontString(nil, 'OVERLAY', 'GameTooltipText')
    --         column:SetText(headerText)
    --         column:SetJustifyH(justify)
    --         -- column:SetPoint('LEFT', mainFrame.header, 'LEFT', xOffset, 0)
    --         -- column:SetWidth(columnWidth)

    --         table.insert(mainFrame.header.columns, column)
    --     end
    -- end

    -- contents
    mainFrame.contents = CreateFrame('Frame', nil, mainFrame.scrollChild)
    mainFrame.contents:SetAllPoints(mainFrame.scrollChild)

    -- hover highlight
    if highlightHover then
        mainFrame.contents.rowHighlight = CreateFrame('Frame', nil, mainFrame.contents)
        local highlightTexture = mainFrame.contents.rowHighlight:CreateTexture(nil, 'OVERLAY')
        highlightTexture:SetAllPoints()
        highlightTexture:SetColorTexture(1, 1, 0, 0.3)
        highlightTexture:SetBlendMode('ADD')
        mainFrame.contents.rowHighlight:Hide()
    end

    -- click highlight
    if highlightClick then
        mainFrame.contents.rowSelectedHighlight = CreateFrame('Frame', nil, mainFrame.contents)
        local highlightTexture = mainFrame.contents.rowSelectedHighlight:CreateTexture(nil, 'OVERLAY')
        highlightTexture:SetAllPoints()
        highlightTexture:SetColorTexture(1, 1, 0, 0.3)
        highlightTexture:SetBlendMode('ADD')
        mainFrame.contents.rowSelectedHighlight:Hide()
    end

    mainFrame.rows = List:new()

    o._mainFrame = mainFrame

    return o
end


---@param data table
function Table:setData(data)
    self._data = data
end


function Table:SetPoint(point, relativeTo, relativePoint, ofsX, ofsY)
    self._mainFrame:SetPoint(point, relativeTo, relativePoint, ofsX, ofsY)
end
